<?php

/**
 * Clear the greydStyles array for the accordion block to prevent double
 * rendering of styles, as they are already saved in the block content.
 * 
 * @since 2.8.0
 * 
 * @param array $greydStyles
 * @param array $block
 * 
 * @return array
 */
function filter_greyd_accordion_styles_object( $greydStyles, $block ) {
	return array();
}
add_filter( 'greyd_styles_block_styles-greyd/accordion', 'filter_greyd_accordion_styles_object', 10, 2 );

/**
 * Render accordion block.
 *
 * @see https://developer.wordpress.org/reference/hooks/render_block/
 *
 * @param string $block_content     The block content about to be appended.
 * @param array  $block             The full block, including name and attributes.
 *
 * @return string $block_content    altered Block Content
 */
function greyd_render_accordion_json_ld_script( $block_content, $block ) {

	if ( $block['blockName'] !== 'greyd/accordion' ) {
		return $block_content;
	}

	$renderStructuredData = isset( $block['attrs']['renderStructuredData'] ) ? $block['attrs']['renderStructuredData'] : false;

	if ( ! $renderStructuredData ) {
		return $block_content;
	}

	$faq = array(
		'@context' => 'https://schema.org',
		'@type'    => 'FAQPage',
		'mainEntity' => array(),
	);
	foreach ( $block['innerBlocks'] as $item ) {
		$question = trim( strip_tags( $item['innerHTML'] ) );

		$answer = '';
		if ( isset( $item['innerBlocks'] ) && is_array( $item['innerBlocks'] ) ) {
			foreach ( $item['innerBlocks'] as $innerBlock ) {
				if ( $innerBlock['blockName'] === 'core/paragraph' ) {
					$answer .= (new \WP_Block( $innerBlock ))->render();
				}
			}
		}

		if (
			empty( $answer )
			|| empty( trim( $answer ) )
			|| trim( $answer ) === '<p></p>'
			|| empty( $question )
			|| empty( trim( $question ) )
			|| trim( $question ) === '<p></p>'
		) {
			continue;
		}
		
		$faq['mainEntity'][] = array(
			'@type' => 'Question',
			'name'  => $question,
			'acceptedAnswer' => array(
				'@type' => 'Answer',
				'text'  => trim( strip_tags( $answer ) ),
			),
		);
	}
	
	if ( ! empty( $faq['mainEntity'] ) ) {
		$block_content .= '<script type="application/ld+json">' . json_encode( $faq ) . '</script>';
	}

	return $block_content;
}

add_filter( 'render_block', 'greyd_render_accordion_json_ld_script', 9, 2 );