<?php

$multiselect = isset( $attributes['multiselect'] ) && $attributes['multiselect'] ? 'true' : 'false';
$showCount = isset( $attributes['showCount'] ) ? (bool)$attributes['showCount'] : false;
$inherit     = isset( $attributes['inherit'] ) ? (bool)$attributes['inherit'] : false;
$label       = isset( $attributes['label'] ) ? $attributes['label'] : '';
$placeholder = isset( $attributes['placeholder'] ) ? $attributes['placeholder'] : '';
$layout      = isset( $attributes['layout'] ) ? $attributes['layout'] : '';
$font_size   = isset( $attributes['font_size'] ) ? $attributes['font_size'] : '';
$style       = isset( $attributes['style'] ) ? $attributes['style'] : '';
$css_class   = isset( $attributes['css_class'] ) ? $attributes['css_class'] : '';
$filterBy    = isset( $attributes['filterBy'] ) ? $attributes['filterBy'] : '';
$title       = isset( $attributes['title'] ) ? $attributes['title'] : '';
$posttype    = isset( $attributes['parentPosttype'] ) ? $attributes['parentPosttype'] : '';

// styles
$greydClass   = isset( $attributes['greydClass'] ) ? $attributes['greydClass'] : wp_unique_id( 'filter_' );
$greydStyles  = isset( $attributes['greydStyles'] ) ? (array)$attributes['greydStyles'] : array();
$customStyles = isset( $attributes['customStyles'] ) && isset( $attributes['customStyles'] ) ? (array)$attributes['customStyles'] : array();


// debug($wrapper_attributes);

$extraClass = '';
if ( isset( $attributes['className'] ) ) {
	if ( strpos( $attributes['className'], 'is-style-prim' ) !== false ) {
		$extraClass = 'button is-style-prim';
	} else if ( strpos( $attributes['className'], 'is-style-sec' ) !== false ) {
		$extraClass = 'button is-style-sec';
	} else if ( strpos( $attributes['className'], 'is-style-trd' ) !== false ) {
		$extraClass = 'button is-style-trd';
	} else if ( strpos( $attributes['className'], 'is-style-tabs' ) !== false ) {
		$extraClass = 'tabs is-style-tabs';
	} else if ( strpos( $attributes['className'], 'is-style-chips' ) !== false ) {
		$extraClass = 'chips is-style-chips';
	} else {
		$extraClass = '';
	}
}

$wrapper_attributes = get_block_wrapper_attributes(
	array( 'class' => $greydClass )
);

global $wp;
$query_vars = $wp->query_vars;

// inherit posttype from query
if ( empty($posttype) && $inherit ) {
	if ( isset( $_GET['post_type'] ) ) {
		$posttype = $_GET['post_type'];
	} else {
		$posttype = get_post_type( get_the_ID() );
	}
}

/**
 * By default, WordPress search supports the URL query parameter 'category_name' for category search.
 * In order to properly support the filter dropdown, we need to change the filterBy value to 'category_name'.
 */
if ( $filterBy === 'categories' || $filterBy === 'category' ) {
	$filterBy = 'category_name';
}

$error_message = '';

$options  = array();
$selected = array();

/**
 * Get options
 */
if ( empty( $posttype ) ) {
	if ( isset( $query_vars['post_type'] ) ) {
		$selected = explode( ',', $query_vars['post_type'] );
	}

	$searchable_posttypes = get_post_types(
		array(
			'exclude_from_search' => false,
		),
		'objects'
	);

	foreach ( $searchable_posttypes as $pt ) {
		$options[ $pt->name ] = $pt->label;
	}
} else {
	$terms = array();

	// post_tag
	if ( $filterBy === 'tag' ) {
		$terms = get_terms(
			array(
				'taxonomy'   => 'post_tag',
				'hide_empty' => true,
			)
		);
	}
	// category
	else if ( $filterBy === 'category' || $filterBy === 'categories' || $filterBy === 'category_name' ) {
		$terms = get_terms(
			array(
				'taxonomy'   => 'category',
				'hide_empty' => true,
			)
		);
	}
	// custom taxonomy
	else {
		$terms = get_terms(
			array(
				'taxonomy'   => $filterBy,
				'hide_empty' => true,
			)
		);
	}

	if ( is_wp_error( $terms ) ) {
		$error_message  = sprintf(
			__( 'Error loading the filter options for %s', 'greyd_hub' ),
			$filterBy
		);
		$error_message .= ': '.$terms->get_error_message();
		$options        = array();
	}

	// map options
	if ( empty( $terms ) ) {
		$options = array();
	} else {
		foreach ( $terms as $term ) {
			if ( is_a( $term, 'WP_Term' ) && isset( $term->name ) && isset( $term->slug ) && $term->count > 0 ) {
				$options[ $term->slug ] = $term->name . ( $showCount ? ' <span class="count">('.$term->count.')</span>' : '' );
			}
		}
	}

	$selected = $inherit && isset( $query_vars[ $filterBy ] ) ? explode( ',', $query_vars[ $filterBy ] ) : array();

	// look for query var in URL param
	if ( $inherit && empty( $selected ) && isset( $_GET[ $filterBy ] ) ) {
		$selected = explode( ',', $_GET[ $filterBy ] );
	}
}

// fix responsive flex alignment
if ( isset( $greydStyles ) &&
	( isset( $greydStyles['alignItems'] ) ||
	isset( $greydStyles['justifyContent'] ) )
) {
	if ( !isset( $greydStyles['flexDirection'] ) || $greydStyles['flexDirection'] !== 'column' ) {
		$greydStyles['justifyContent'] = $greydStyles['alignItems'];
		unset( $greydStyles['alignItems'] );
	} else if ( $greydStyles['flexDirection'] === 'column' && isset( $greydStyles['justifyContent'] ) ) {
		$greydStyles['alignItems'] = $greydStyles['justifyContent'];
		unset( $greydStyles['justifyContent'] );
	}
}

\greyd\blocks\render::enqueue_custom_style(
	".{$greydClass}",
	$greydStyles
);

\greyd\blocks\render::enqueue_custom_style(
	".{$greydClass} .greyd_filter_button",
	$customStyles,
	array(
		'pseudo_active' => '.is-active',
		'important'     => true,
	)
);

?>

<div <?php echo $wrapper_attributes; ?> data-multiselect="<?php echo $multiselect; ?>">

	<?php if ( !empty( $error_message ) ) : ?>
		<div class="error-message"><?php echo $error_message; ?></div>
	<?php endif; ?>

	<input type="hidden" name="<?php echo $filterBy; ?>" value="<?php echo implode( ',', $selected ); ?>" />
	<?php foreach ( $options as $val => $text ) : ?>
		<div class="greyd_filter_button <?php echo $extraClass; ?>">
			<input type="radio" value="<?php echo $val; ?>" />
			<div class="option <?php echo $css_class; ?>"></div>
			<span class="label"><?php echo $text; ?></span>
		</div>
	<?php endforeach; ?>

</div>
