/**
 * Filter buttons
 */
document.addEventListener( "DOMContentLoaded", function () {

	const filterWrappers = document.querySelectorAll( ".wp-block-greyd-search-filter-buttons" );

	filterWrappers.forEach( filterWrapper => {
		
		const filterButtons = filterWrapper.querySelectorAll( ".greyd_filter_button" );
		const isMultiselect = filterWrapper.getAttribute( "data-multiselect" ) === "true" ? true : false;
		const hiddenInput = filterWrapper.querySelector( "input[type='hidden']" );

		if ( hiddenInput.value.length > 1 ) {
			const values = hiddenInput.value.split( "," );
			filterButtons.forEach( button => {
				const optionValue = button.querySelector( "input[type='radio']" ).value;
				if ( values.includes( optionValue ) ) {
					button.classList.add( "is-active" );
				}
			} );
		}

		if ( !filterButtons.length ) return;

		filterButtons.forEach( filterButton => {
			const  optionValue = filterButton.querySelector( "input[type='radio']" ).value;

			filterButton.addEventListener( "click", function () {
				
				if ( !isMultiselect ) {
					
					const isSelected = filterButton.classList.contains( "is-active" );
					hiddenInput.value = isSelected ? '' : optionValue;
					
					// trigger change event
					hiddenInput.dispatchEvent( new Event( "change" ) );

					filterButtons.forEach( button => {
						if ( button !== this ) {
							button.classList.remove( "is-active" );
						}
					} );
				} else {
					let values = hiddenInput.value;
					values = values.split( "," ).filter( value => value.length > 0 );

					let index = values.indexOf( optionValue );

					if ( index > -1 ) {
						values.splice( index, 1 );
					} else {
						values.push( optionValue );
					}
					
					hiddenInput.value = values.join( "," );

					// trigger change event
					hiddenInput.dispatchEvent( new Event( "change" ) );
				}

				filterButton.classList.toggle( "is-active" );
			} );
		});
	});
} );